using System;
using System.Text;
using System.Configuration;
using System.Web;
using System.Web.Security;

namespace ASPItalia.com.Scripts.MultiLogin
{

	public class MultiLoginModule : IHttpModule
	{
		private static MultiLoginConfig _settings = null;

		public MultiLoginModule()
		{

		}

		public static MultiLoginConfig Settings
		{
			get
			{
				if (_settings == null)
				{
					// Carico le config
					_settings = (MultiLoginConfig)ConfigurationSettings.GetConfig("multiLogin");
				}
				return _settings;
			}
		}
		

		public void Init(HttpApplication context)
		{
			context.AuthorizeRequest += new EventHandler(context_AuthorizeRequest);
			context.EndRequest += new EventHandler(context_EndRequest);		
		}

		public void Dispose()
		{
			
		}

		private void context_EndRequest(object sender, EventArgs e)
		{
			HttpContext context = HttpContext.Current;
			// Controllo lo StatusCode
			if (context.Response.StatusCode == 401)
			{
				// Certo la il path nel config
				if (Settings.Page == null) return;
				string path = context.Request.PhysicalPath.ToLower();;
				foreach (PageLoginType page in Settings.Page)
				{
					string l = context.Server.MapPath(page.LoginUrl).ToLower();
					string p = context.Server.MapPath(page.Path).ToLower();
					// Se la pagina richiesta  nella lista di path
					if (p == path)
					{
						// Effettuo il redirect nuovo
						redirectToWithReturnUrl(page.LoginUrl, context);
						break;
					}

				}
			}
		}


		private void redirectToWithReturnUrl(string loginUrl, HttpContext context)
		{
			StringBuilder sb = new StringBuilder(loginUrl);

			// Preparo il link
			int i = loginUrl.IndexOf("?");
			if (i > 0)
				sb.Append('&');
			else
				sb.Append('?');

			// Aggiungo la pagina richiesta come parametro returnUrl
			// come fa gi l'auth form
			sb.Append("returnUrl=");
			sb.Append(HttpUtility.UrlEncode(context.Request.RawUrl));

			// Effettuo il redirect
			context.Response.Redirect(sb.ToString(), false);
		}

		private void context_AuthorizeRequest(object sender, EventArgs e)
		{
			HttpContext context = HttpContext.Current;

			// Certo la il path nel config
			if (Settings.Page == null) return;
			string path = context.Request.PhysicalPath.ToLower();;
			foreach (PageLoginType page in Settings.Page)
			{
				string l = context.Server.MapPath(page.LoginUrl).ToLower();
				// Se il file richiesto  una delle pagine di login
				// allora faccio saltare l'autorizzazione al UrlAuthorizationModule
				if (l == path)
				{
					context.SkipAuthorization = true;
					break;
				}
			}
		}
	}
}